<?php

namespace Drupal\alpine_js;

use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * A service to post manipulate the attaches js files.
 */
class AlpineAssetService {

  /**
   * Alpine_JS configuration.
   */
  protected Array $alpineConfig;

  /**
   * LibraryDiscoveryInterface.
   */
  protected LibraryDiscoveryInterface $librariesDiscovery;

  /**
   * Is AlpineJS Found?
   */
  protected bool $alpinejsFound = FALSE;

  /**
   * Libraries that need AlpineJS.
   */
  protected array $alpineLibraries = [];

  protected bool $alpinejsCSP = FALSE;

  /**
   * The Libraries that do not need AlpineJS.
   */
  protected array $drupalLibraries = ['header' => [],'footer' => []];

  /**
   * The re-sorted AlpineJS libraries.
   */
  protected array $alpineDependencies = [];

  /**
   * Default options for libraries that don't have them.
   */
  protected array $defaultOptions = [
    'browsers' => [],
    'cache' => TRUE,
    'group' => JS_LIBRARY,
    'preprocess' => TRUE,
    'weight' => 0,
  ];

  /**
   * Scope of where to put the AlpineJS libraries, footer or header.
   */
  private string $alpineScope;

  /**
   * Construct a new instance.
   */
  public function __construct(ConfigFactoryInterface $configFactory, LibraryDiscoveryInterface $librariesDiscovery) {
    $this->librariesDiscovery = $librariesDiscovery;
    $this->alpineConfig = $configFactory->get('alpine_js.settings')->get();
    $this->alpineScope = ($this->alpineConfig['footer']) ? 'footer' : 'header';
    $this->alpinejsCSP = $this->alpineConfig['csp'] ?? false;
  }

  /**
   * Get the Base Alpine libraries as defined by this module.
   */
  public function getAlpineBaseLibraries(): array {
    $alpineBaseLibraries = [];
    foreach ($this->librariesDiscovery->getLibrariesByExtension('alpine_js') as $library => $assetsInfo) {
      foreach ($assetsInfo['js'] as $definition) {

        $definition['scope'] = $this->alpineScope;
        $path = $assetInfo['source'] ?? $definition['data'];
        $definition += $this->defaultOptions;
        $alpineBaseLibraries[$library][$path] = $definition;
      }
    }
    return $alpineBaseLibraries;
  }

  /**
   * Traverse all the sources and split out the ones that  depend on AlpineJS.
   */
  public function findAndSplitAlpineLibraries(&$sources): void {

    foreach ($sources as $fileName => $definition) {
      if (array_key_exists('attributes', $definition) && array_key_exists('alpinejs', $definition['attributes'])) {
        $definition['scope'] = $this->alpineScope;

        $this->alpineLibraries[$fileName] = $definition;
        unset($sources[$fileName]);
        $this->alpinejsFound = TRUE;
      }
      else {
        $scope = $definition['scope'];
        $this->drupalLibraries[$scope][$fileName] = $definition;
      }
    }
    if ($this->alpinejsFound) {
      $this->addAlpineToLibraries();
      $sources = array_merge_recursive($this->drupalLibraries['header'], $this->drupalLibraries['footer']);
    }
  }

  /**
   * Add a source that needs AlpineJS to the stack.
   */
  protected function addToAlpineDependencies(array $source, int $group, float &$weight): void {
    foreach ($source as $sourcePath => $definition) {
      $weight += 0.001;
      $definition['group'] = $this->defaultOptions['group'];
      // Override the preprocess, so the bundler actually want to bundle.
      $definition['preprocess'] = TRUE;
      // Remove the alpineJS attribute.
      unset($definition['attributes']['alpinejs']);
      $definition['weight'] = $weight;
      $this->alpineDependencies[$sourcePath] = $definition;
    }
  }

  /**
   * Add AlpineJS + dependent libraries to the end of the sources.
   */
  public function addAlpineToLibraries() : void {

    $alpineBaseLibraries = $this->getAlpineBaseLibraries();

    // Remove the base libraries if the ended up somewhere in the
    // Drupal attached libraries.
    foreach ($alpineBaseLibraries['alpine'] as $jsName => $info) {
      if (array_key_exists($jsName, $this->drupalLibraries['header'])) {
        unset($this->drupalLibraries['header'][$jsName]);
      }
      if (array_key_exists($jsName, $this->drupalLibraries['footer'])) {
        unset($this->drupalLibraries['footer'][$jsName]);
      }
    }
    $lastItem = array_key_last($this->drupalLibraries[$this->alpineScope]);
    $weight = $lastItem['weight'] ?? 10;
    $group = -100;

    $this->addToAlpineDependencies($this->alpineLibraries, $group, $weight);

    // Add AlpineJS as the very last item in the chain, as it contains the
    // alpine.init call;.
    if (TRUE === (bool) $this->alpineConfig['bridge']) {
      $this->addToAlpineDependencies($alpineBaseLibraries['drupalbridge'], $group, $weight);
    }
    if ($this->alpinejsCSP){
      $this->addToAlpineDependencies($alpineBaseLibraries['alpine-csp'], $group, $weight);
    } else {
     $this->addToAlpineDependencies($alpineBaseLibraries['alpine'], $group, $weight);  
    }
    $this->drupalLibraries[$this->alpineScope] += $this->alpineDependencies;
  }

}
